/*
*  program name: Rubrica <http://digilander.iol.it/nfragale/download.html>
*  file: card.c
*  
*  Copyright (C) 2000-2002 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>
#include <gconf/gconf-client.h>

#include "main.h"
#include "callbacks.h"
#include "card.h"
#include "view.h"
#include "rubrica.h"
#include "rubrica_type.h"
#include "dialoghi.h"
#include "prefer.h"
#include "prefer-notify.h"
#include "librubrica.h"

GtkWidget *tree_view;
GtkWidget *notebook;
gboolean confirm_remove = TRUE;
gint pages = 0;


GtkWidget *rubrica_card_get_notebook(void)
{
  return (GtkWidget *) notebook;
}

gint rubrica_card_get_current_notebook_page(void)
{
  return gtk_notebook_get_current_page(GTK_NOTEBOOK (notebook));
}


GtkWidget *rubrica_card_get_nth_parent(gint page)
{
  return (GtkWidget *) gtk_notebook_get_nth_page(GTK_NOTEBOOK (notebook), page);
}


GtkTreeView *rubrica_card_get_view(GtkWidget *child)
{
  g_return_val_if_fail(child != NULL, NULL);

  return GTK_TREE_VIEW(g_object_get_data(G_OBJECT(child), "tree_view"));
}


GtkTreeView *rubrica_card_get_current_view(void)
{
  GtkWidget *child;

  child = rubrica_card_get_current_child();

  return GTK_TREE_VIEW(g_object_get_data(G_OBJECT(child), "tree_view"));
}


void rubrica_card_set_view(GtkWidget *view)
{
  tree_view = view; 
} 


GtkWidget *rubrica_card_get_current_child(void)
{
  GtkWidget *child;
  gint page;

  page = rubrica_card_get_current_notebook_page();
  child = gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), page);

  return (GtkWidget *) child;
}


gint rubrica_card_get_pages(void)
{
  return (gint) pages;
}


void rubrica_card_update_tab_name(GtkWidget *child, gchar *str)
{
  gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(notebook), child, str);
}


Rubrica *rubrica_card_get_rubrica_from_view(GtkTreeView *view)
{
  return (Rubrica *) g_object_get_data(G_OBJECT(view), "rubrica");
}


RubricaItem* rubrica_card_get_nth_item(GtkTreeView *view, gint n)
{
  RubricaItem *item;
  GtkTreeModel *model;
  GtkTreeIter iter; 
  gchar *string;
  gboolean bool;

  item   = g_malloc(sizeof(RubricaItem));

  model  = gtk_tree_view_get_model(GTK_TREE_VIEW(view));
  string = g_strdup_printf("%d", n);
  
  bool = gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(model), &iter, string);
  g_free(string);

  if (bool)
    gtk_tree_model_get (GTK_TREE_MODEL(model), &iter, RUBRICA_COLUMN, &item, -1);
  else
    return NULL;

  return (RubricaItem *) item;
}


/**
 * rubrica_card_interface_creation
 *
 * create the interface (a notebook) for card's window. 
 * the notebook's child is a scrolled window, 
 * and a tree view is packed into scrolle. Do not delete this notebook
 *
 * Return: notebook  
 */ 
GtkWidget *rubrica_card_interface_creation(RubricaInfoChild *info)
{
  notebook = gtk_notebook_new();

  gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_BOTTOM);
  gtk_notebook_set_scrollable(GTK_NOTEBOOK (notebook), TRUE);
  gtk_widget_show(notebook);

  info->child = notebook;

  return (GtkWidget *) notebook;
}



GtkTreeModel *rubrica_card_create_model(void)
{
  GtkListStore *model;

  /*      create list store 
   */
  model = gtk_list_store_new(COLUMNS, 
			     G_TYPE_STRING,       /* card       */
			     G_TYPE_STRING,       /* first name */ 
			     G_TYPE_STRING,       /* last name  */ 
			     G_TYPE_STRING,       /* profession */  
			     G_TYPE_STRING,       /* city       */
			     G_TYPE_STRING,       /* country    */
			     G_TYPE_STRING,       /* web        */
			     G_TYPE_STRING,       /* email      */ 
			     G_TYPE_STRING,       /* telephone  */ 
			     G_TYPE_STRING,       /* company    */  
			     G_TYPE_STRING,       /* assigment  */
			     G_TYPE_STRING,       /* categories */
			     G_TYPE_POINTER);     /* rubrica's data */      

  return GTK_TREE_MODEL(model);
}


GtkWidget *rubrica_card_new_view(GtkTreeModel *model)
{
  GtkWidget *scrollwin;
  GtkWidget *view;
  GtkTreeSelection *select;

  scrollwin = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrollwin), 
				      GTK_SHADOW_ETCHED_IN);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin),
				 GTK_POLICY_AUTOMATIC, 
				 GTK_POLICY_AUTOMATIC);

  /*     create a new view using the model and connect the select
	 widget to changed signal
  */
  view = gtk_tree_view_new_with_model(model);
  select = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
  rubrica_card_set_view(view);

  g_object_unref(G_OBJECT(model));

  /*    set view's properties
  */
  gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
  gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(view), TRUE);
  gtk_tree_view_set_reorderable (GTK_TREE_VIEW (view), TRUE);
  gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW (view), TRUE);

  rubrica_card_add_columns(GTK_TREE_VIEW(view));

  gtk_container_add(GTK_CONTAINER(scrollwin), view);
  g_object_set_data(G_OBJECT(scrollwin), "tree_view", view);
  gtk_widget_show(view);

  g_signal_connect(G_OBJECT(select), "changed", 
		   G_CALLBACK(rubrica_card_row_selected), view);
  g_signal_connect(G_OBJECT(view), "row_activated", 
		   G_CALLBACK(rubrica_card_row_activated), view);
  
  return scrollwin;
}

void rubrica_card_set_column_attributes(GConfClient *client, GtkTreeView *view, 
					gchar *key, gint col)
{
  GtkTreeViewColumn *column;
  
  column = gtk_tree_view_get_column(view, col);
  gtk_tree_view_column_set_resizable(column, TRUE);
//  gtk_tree_view_column_set_clickable(column, TRUE);
  gtk_tree_view_column_set_reorderable(column, TRUE);

  rubrica_notify_column_config(client, column, key);
}


void rubrica_card_add_columns(GtkTreeView *view)
{
  GtkCellRenderer *renderer;
  GtkTreeModel *model;
  gint col;
  GConfClient *client;

  model  = gtk_tree_view_get_model(view);
  client = gconf_client_get_default(); // rubrica_get_client();
  
  /*     Create the column of list
  */
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes(view, /* tree view */
							 -1,  /* column position (-1 = last) */
							 _("Card"),   /* column title */
							 renderer,    /* column cell */
							 "text",      /* column attribute */
							 CARD_COLUMN, /* column */
							 NULL);       /* no more attribute */

  rubrica_card_set_column_attributes(client, view, 
				     RUBRICA_KEY_SHOW_CARD_COLUMN_CARD, col-1);

  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes(view, -1, _("First Name"), renderer,
							 "text", FIRST_NAME_COLUMN, NULL);
  rubrica_card_set_column_attributes(client, view, 
				     RUBRICA_KEY_SHOW_CARD_COLUMN_FIRST_NAME, col-1);
  
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Last Name"), renderer,
							  "text", LAST_NAME_COLUMN, NULL);
  rubrica_card_set_column_attributes(client, view, 
				     RUBRICA_KEY_SHOW_CARD_COLUMN_LAST_NAME, col-1);  

  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Profession"), renderer,
							  "text", PROFESSION_COLUMN, NULL);
   rubrica_card_set_column_attributes(client, view, 
				      RUBRICA_KEY_SHOW_CARD_COLUMN_PROFESSION, col-1); 

  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("City"), renderer,
							  "text", CITY_COLUMN, NULL);
  rubrica_card_set_column_attributes(client, view,
				     RUBRICA_KEY_SHOW_CARD_COLUMN_CITY, col-1);
  
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Country"), renderer,
							  "text", COUNTRY_COLUMN, NULL);
  rubrica_card_set_column_attributes(client, view, 
				     RUBRICA_KEY_SHOW_CARD_COLUMN_COUNTRY, col-1);
  
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Web"), renderer,
							  "text", WEB_CARD_COLUMN, NULL);
  rubrica_card_set_column_attributes(client, view, 
				     RUBRICA_KEY_SHOW_CARD_COLUMN_WEB, col-1);
  
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Email"), renderer,
							  "text", EMAIL_CARD_COLUMN, NULL);
  rubrica_card_set_column_attributes(client, view, 
				     RUBRICA_KEY_SHOW_CARD_COLUMN_EMAIL, col-1);
  
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Telephone"), renderer,
							  "text", TELEPHONE_COLUMN, NULL);
  rubrica_card_set_column_attributes(client, view, 
				     RUBRICA_KEY_SHOW_CARD_COLUMN_TELEPHONE, col-1);
  
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Company"), renderer,
							  "text", COMPANY_COLUMN, NULL);
  rubrica_card_set_column_attributes(client, view,
				     RUBRICA_KEY_SHOW_CARD_COLUMN_COMPANY, col-1);
  
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Assigment"), renderer,
							  "text", ASSIGMENT_COLUMN, NULL);
  rubrica_card_set_column_attributes(client, view,
				     RUBRICA_KEY_SHOW_CARD_COLUMN_ASSIGMENT, col-1);  

  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Categories"), renderer,
							  "text", CATEGORIES_COLUMN, NULL);  
  rubrica_card_set_column_attributes(client, view, 
				     RUBRICA_KEY_SHOW_CARD_COLUMN_CATEGORIES, col-1);
}


/*  add into addressbook's notebook a new page with the 
    content of file "filename"

    return the number of the new page
*/

gint 
rubrica_card_add_page_view(gchar *filename)
{
  Rubrica   *rubrica;       /* the new addressbook item */

  GtkWidget *tab_label;
  GtkWidget *tab_box;
  GtkWidget *green;
  GtkWidget *red;

  GtkTreeModel *model;
  GtkWidget *child;      /* notebook child, this will contain the tree_view */
  GtkTreeView *view;     /* the cards list */
  gint   page;           /* the # of the new notebook's page */
  gchar *path; 
  gchar *fname = NULL;


  if (filename)
    {
      path  = g_dirname(filename);
      fname = g_strdup(g_basename(filename));
    }
  else
    path = rubrica_preferences_get_home();

  /*      get the number of this new page 
   */
  page = rubrica_card_get_pages();

  /*    build new addressbook. 
	It will be related to new notebook page  
  */
  rubrica = rubrica_new();

  /*    make notebook page's child 
   */
  model = rubrica_card_create_model();
  child = rubrica_card_new_view(model);
  view  = rubrica_card_get_view(child); 
  gtk_widget_show(child);

  tab_box = rubrica_lib_default_box_new(HBOX);  

  if (!fname)
    fname = _("no name");
  tab_label = gtk_label_new (fname);  
  gtk_widget_show(tab_label);
  
  green = rubrica_lib_image_new("ledgreen.png");
  red   = rubrica_lib_image_new("ledred.png");
  gtk_widget_hide(red);
  
  gtk_box_pack_start(GTK_BOX(tab_box), green, FALSE, FALSE, 1);
  gtk_box_pack_start(GTK_BOX(tab_box), red,   FALSE, FALSE, 1);
  gtk_box_pack_start(GTK_BOX(tab_box), tab_label, TRUE, TRUE, 1);

  /*      store infos about this addressbook/page 
   */
  rubrica->name   = (fname ? g_strdup(fname) : NULL);
  rubrica->path   = (path  ? g_strdup(path)  : NULL);
  rubrica->parent = child;
  rubrica->view   = view;
  rubrica->page   = page;
//  gtk_tree_model_get_iter_from_string(model, &rubrica->iter, "0");

  g_object_set_data(G_OBJECT(child), "rubrica", rubrica);
  g_object_set_data(G_OBJECT(view), "rubrica", rubrica);
  g_object_set_data(G_OBJECT(tab_box), "green", green);
  g_object_set_data(G_OBJECT(tab_box), "red", red);

  /*      append a new addressbook view 
	  and increase the pages number 
   */
  rubrica_card_append_page(child, tab_box); 
//  rubrica_make_this_rubrica_current_rubrica(rubrica);

  /*      return the number of the new page 
   */
  return (gint) page;  
}


void rubrica_card_append_page(GtkWidget *child, GtkWidget *tabbox)
{
  pages++;
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child, tabbox);  
  gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), -1);  
}


void rubrica_card_remove_page(gint page)
{
  gtk_notebook_remove_page(GTK_NOTEBOOK(notebook), page);

  pages--;
}


void rubrica_card_append_item(Rubrica *rubrica, RubricaItem *item)
{
  GtkTreeView *view;
  GtkTreeIter iter;
  GtkTreeModel *model;

  gchar *web, *email, *number;

  g_return_if_fail(item != NULL);

  view = GTK_TREE_VIEW(rubrica_get_view(rubrica));
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));

  if (item->net.web && item->net.web->data)
    web = g_strdup(item->net.web->data);
  else
    web = "";

  if (item->net.email && item->net.email->data)
    email = g_strdup(item->net.email->data);
  else
    email = "";

  if (item->phone.telephone && item->phone.telephone->data)
    {
      TelNum *tn;
      
      tn = item->phone.telephone->data;
      number = g_strdup(tn->number);
    }
  else
    number = "";
  
  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set(GTK_LIST_STORE(model), &iter,
		     CARD_COLUMN,       item->card, 
		     FIRST_NAME_COLUMN, item->personal.first, 
		     LAST_NAME_COLUMN,  item->personal.last,
		     PROFESSION_COLUMN, item->personal.profession,
		     CITY_COLUMN,       item->address.city,
		     COUNTRY_COLUMN,    item->address.country,
		     WEB_CARD_COLUMN,   web,
		     EMAIL_CARD_COLUMN, email,
		     TELEPHONE_COLUMN,  number,
		     COMPANY_COLUMN,    item->company.name,
		     ASSIGMENT_COLUMN,  item->work.assigment,
		     CATEGORIES_COLUMN, item->group.label, 
		     RUBRICA_COLUMN,    item,
		     -1);  
}


void rubrica_card_insert_item(Rubrica *rubrica, RubricaItem *item, gint index)
{
  GtkTreeView *view;
  GtkTreeIter iter;
  GtkTreeModel *model;

  gchar *web, *email, *number;
  
  g_return_if_fail(item != NULL);

  view = GTK_TREE_VIEW(rubrica_get_view(rubrica));
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree_view));

  if (item->net.web && item->net.web->data)
    web = g_strdup(item->net.web->data);
  else
    web = "";

  if (item->net.email && item->net.email->data)
    email = g_strdup(item->net.email->data);
  else
    email = "";

  if (item->phone.telephone && item->phone.telephone->data)
    {
      TelNum *tn;
      
      tn = item->phone.telephone->data;
      number = g_strdup(tn->number);
    }
  else
    number = ""; 

  gtk_list_store_insert(GTK_LIST_STORE(model), &iter, index);
  gtk_list_store_set(GTK_LIST_STORE(model), &iter,
		     CARD_COLUMN,       item->card, 
		     FIRST_NAME_COLUMN, item->personal.first, 
		     LAST_NAME_COLUMN,  item->personal.last,
		     PROFESSION_COLUMN, item->personal.profession,
		     CITY_COLUMN,       item->address.city,
		     COUNTRY_COLUMN,    item->address.country,
		     WEB_CARD_COLUMN,   web,
		     EMAIL_CARD_COLUMN, email,
		     TELEPHONE_COLUMN,  number,
		     COMPANY_COLUMN,    item->company.name,
		     ASSIGMENT_COLUMN,  item->work.assigment,
		     CATEGORIES_COLUMN, item->group.label, 
		     RUBRICA_COLUMN,    item,
		     -1);  
}


void rubrica_card_modify_item(Rubrica *rubrica, RubricaItem *r, gint index)
{
  confirm_remove = FALSE;
  
  if (rubrica_card_remove_selected_item(rubrica))
    rubrica_card_insert_item(rubrica, r, index);  

  confirm_remove = TRUE;  
}


gboolean rubrica_card_remove_selected_item(Rubrica *rubrica)
{
  GtkTreeIter iter;
  GtkTreeView *view;
  GtkTreeModel *model;
  GtkTreeSelection *selection;  

  view  = GTK_TREE_VIEW(rubrica_get_view(rubrica));
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));

  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
    {
      if (confirm_remove)
	{
	  GtkWidget *dialog;
	  gchar *card;
	  gchar *str;
	  gint choose;
	  
	  gtk_tree_model_get (model, &iter, CARD_COLUMN, &card, -1);
	  str = g_strdup_printf(_("Do you want delete the %s card?"), card);

	  dialog = rubrica_dialog_query(str, GTK_BUTTONS_YES_NO);
	  gtk_widget_show(dialog);
	  g_free(str);
	  
	  choose = gtk_dialog_run(GTK_DIALOG(dialog));
	  gtk_widget_destroy(dialog);

	  switch(choose)
	    {
	    case GTK_RESPONSE_YES:
	      break;
	      
	    case GTK_RESPONSE_NO:
	    default:
	      return FALSE;
	      break;
	    }
	}
      gtk_list_store_remove(GTK_LIST_STORE(model), &iter);      
      
      return TRUE;
    }
  else
    {
      rubrica_dialog_message(_("I can't continue if you don't select a card"), 
			     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK);
      
      return FALSE;
    }
}

gboolean 
rubrica_card_free_memory(GtkTreeModel *model, GtkTreePath *path, 
			 GtkTreeIter *iter, gpointer data)
{
  RubricaItem *item;
  Rubrica *rubrica = data;
  
  if (rubrica_is_empty(rubrica))
    return TRUE;
  
  gtk_tree_model_get(model, iter, RUBRICA_COLUMN, &item, -1);

  return FALSE;
}


RubricaItem *rubrica_card_item_cut(Rubrica *rubrica)
{
  RubricaItem *item = NULL;
  GtkTreeIter iter;
  GtkTreeView *view;
  GtkTreeModel *model;
  GtkTreeSelection *selection;
  
  view      = GTK_TREE_VIEW(rubrica_get_view(rubrica));
  model     = gtk_tree_view_get_model(view);
  selection = gtk_tree_view_get_selection(view);
  
  if (gtk_tree_selection_get_selected(selection, NULL, &iter))    
    {
      gtk_tree_model_get (model, &iter, RUBRICA_COLUMN, item, -1);
      gtk_list_store_remove(GTK_LIST_STORE(model), &iter);      
    }
  else
    rubrica_dialog_message(_("I can't continue if you don't select a card"), 
			   GTK_MESSAGE_ERROR, GTK_BUTTONS_OK);       
    
  return item;
}


RubricaItem *rubrica_card_item_copy(Rubrica *rubrica)
{
  RubricaItem *item = NULL;
  GtkTreeIter iter;
  GtkTreeView *view;
  GtkTreeModel *model;
  GtkTreeSelection *selection;
  
  view      = GTK_TREE_VIEW(rubrica_get_view(rubrica));
  model     = gtk_tree_view_get_model(view);
  selection = gtk_tree_view_get_selection(view);
  
  if (gtk_tree_selection_get_selected(selection, NULL, &iter))    
    {
      gtk_tree_model_get (model, &iter, RUBRICA_COLUMN, item, -1);
    }
  else
    rubrica_dialog_message(_("I can't continue if you don't select a card"), 
			   GTK_MESSAGE_ERROR, GTK_BUTTONS_OK);       
    
  return item;
}




/***
 *               CALLBACKS
 ***/

void rubrica_card_row_selected(GtkTreeSelection *select, gpointer data)
{
  GtkTreeModel *model = gtk_tree_view_get_model(data);
  Rubrica *rubrica;
  RubricaItem *item;
  GtkTreePath *path;
  GtkTreeIter iter;
  gint index;
  
  rubrica = rubrica_get_current_rubrica();

  if (gtk_tree_selection_get_selected(select, &model, &iter))
    {
      gtk_tree_model_get(model, &iter, RUBRICA_COLUMN, &item, -1);
      path = gtk_tree_model_get_path(model, &iter);
      
      if (gtk_tree_selection_path_is_selected(select, path))
      {
	index = gtk_tree_path_get_indices(path)[0];
	rubrica_set_index(rubrica, index);
      }

      rubrica_buffer_set_item(item);
      rubrica_view_show_card_item(item);
    }
}

void rubrica_card_row_activated(GtkTreeView *tree_view, GtkTreePath *path,
				GtkTreeViewColumn *column)
{
  GtkTreeIter iter;
  GtkTreeModel *model;
  RubricaItem *item;
  gint index;

  model = gtk_tree_view_get_model (tree_view);
  
  gtk_tree_model_get_iter (model, &iter, path);
  index = gtk_tree_path_get_indices (path)[0];
  gtk_tree_model_get (model, &iter, RUBRICA_COLUMN, &item, -1);

  rubrica_modify_card_item(item, index);
}
