/*
*  Rubrica
*  file: librubrica.h
*
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef __LIBRUBRICA_H
#define __LIBRUBRICA_H

#include <gtk/gtk.h>
#include "rubrica_type.h"

/* contenitori
 */
void rubrica_lib_pack(GtkWidget *box, GtkWidget *child, gboolean start);
void rubrica_lib_add(GtkWidget *container, GtkWidget *child);

void rubrica_lib_add_tooltip(GtkTooltips *tooltip);
void rubrica_lib_enable_tooltips(void);
void rubrica_lib_disable_tooltips(void);

void rubrica_lib_add_pixmap_directory(const gchar *directory);
gchar* rubrica_lib_find_pixmap_file (const gchar *filename);
void rubrica_lib_set_atk_action_description (AtkAction *action, const gchar *action_name,
					     const gchar *description);

GtkWidget *rubrica_lib_default_box_new(RubricaBoxType type);
GtkWidget *rubrica_lib_box_new(RubricaBoxType type, gboolean homogeneous, 
			       gint spacing, guint border_width);
GtkWidget *rubrica_lib_frame_new(gchar *str);
GtkWidget *rubrica_lib_image_new(gchar *pixfile);
GdkPixbuf *rubrica_lib_pixbuf_new(gchar *filename);
GtkWidget *rubrica_lib_label_new(gchar *str);
GtkWidget *rubrica_lib_entry_new(void);
GtkWidget *rubrica_lib_check_button_new(gchar *str);
GtkWidget *rubrica_lib_buttonbox_new(RubricaBoxType type);
GtkWidget *rubrica_lib_scrolled_window_new(void);
GtkWidget *rubrica_lib_pixbutton_new(gchar *str, gchar *pixfile);
GtkWidget *rubrica_lib_table_new(gint row, gint column, gboolean homogeneous, 
				 guint border, guint row_spacing, guint col_spacing);

GtkWidget *rubrica_lib_pack_in_box_new_frame(GtkWidget *box, gchar *str, gboolean packstart);
GtkWidget *rubrica_lib_pack_in_box_check_button_new(GtkWidget *box, gchar *str); 
GtkWidget *rubrica_lib_pack_in_box_image_new(GtkWidget *box, gchar *pixfile, 
					      gboolean packstart);

GtkWidget *rubrica_lib_pack_in_box_new_box(GtkWidget *box, RubricaBoxType type, 
					   gboolean homogeneous, gint spacing, guint border,
					   gboolean packstart);
GtkWidget *rubrica_lib_pack_in_frame_new_box(GtkWidget *frame, RubricaBoxType type, 
					     gboolean homogeneous, gint spacing, guint border);


GtkWidget *rubrica_lib_pack_in_box_boxed_label_entry_pix_new(GtkWidget *box, 
							     RubricaBoxType type,
							     gboolean homogeneous,
							     gchar *str, gchar *pixfile, 
							     gboolean first,
							     gboolean packstart);

GtkWidget *rubrica_lib_pack_in_box_boxed_label_option_menu_new(GtkWidget *container, 
							       gchar *str, 
							       RubricaOptionType *option_list,
							       gint option_len,
							       RubricaBoxType type,
							       gboolean packstart);




GtkWidget *rubrica_lib_pack_in_box_table_new(GtkWidget *box, 
					     gint row, gint column, 
					     gboolean homogeneous, 
					     guint border, 
					     guint row_spacing, 
					     guint col_spacing);


GtkWidget *rubrica_lib_pack_in_frame_table_new(GtkWidget *frame, 
					       gint row, gint column, 
					       gboolean homogeneous, 
					       guint border, 
					       guint row_spacing, 
					       guint col_spacing);


void rubrica_lib_tabled_widget(GtkWidget *container, 
			       GtkWidget *child, 
			       guint left_attach, 
			       guint right_attach, 
			       guint top_attach, 
			       guint bottom_attach);

GtkWidget *rubrica_lib_tabled_label_new(GtkWidget *container, 
					gchar *str, 
					guint left_attach,
					guint right_attach, 
					guint top_attach, 
					guint bottom_attach);

GtkWidget *rubrica_lib_tabled_entry_new(GtkWidget *container, 
					guint left_attach, 
					guint right_attach, 
					guint top_attach, 
					guint bottom_attach);

GtkWidget *rubrica_lib_tabled_check_new(GtkWidget *container, 
					gchar *str, 
					guint left_attach, 
					guint right_attach, 
					guint top_attach, 
					guint bottom_attach);

GtkWidget *rubrica_lib_tabled_date_new(GtkWidget *container, 
				       guint left_attach, 
				       guint right_attach, 
				       guint top_attach, 
				       guint bottom_attach);

GtkWidget *rubrica_lib_tabled_radio_new(GtkWidget *container, 
					GSList *group, 
					gchar *str,  
					guint left_attach, 
					guint right_attach,  
					guint top_attach, 
					guint bottom_attach); 

GtkWidget *rubrica_lib_tabled_pixmap_new(GtkWidget *container, 
					 gchar *pixfile,  
					 guint left_attach,
					 guint right_attach,  
					 guint top_attach,
					 guint bottom_attach); 

GtkWidget *rubrica_lib_pack_in_box_widgets(RubricaBoxType type, 
					   GtkWidget* widget1, 
					   GtkWidget* widget2, 
					   gboolean pack_start);

GtkWidget *rubrica_lib_tabled_text_view(GtkWidget *table, GtkWidget *text_view,
					guint left_attach, guint right_attach, 
					guint top_attach, guint bottom_attach);

/*  text view
*/
GtkWidget *rubrica_lib_text_view_new(void);
void rubrica_lib_set_text_in_view(GtkWidget *text_view, gchar *buffer);
void rubrica_lib_clean_text_view(GtkWidget *text_view);


/*   tree view
*/
GtkWidget *rubrica_lib_tree_view_new_with_model(GtkTreeModel* model);
GtkWidget *rubrica_lib_pack_in_frame_tree_view_new_with_model(GtkWidget *frame, 
							      GtkTreeModel* model);
GtkWidget *rubrica_lib_pack_in_box_tree_view_new_with_model(GtkWidget *container,
							    GtkTreeModel* model);
void rubrica_lib_add_columns(GtkTreeView *view, RubricaColumnsType type);
GList *rubrica_lib_get_all_data(GtkWidget *tree, RubricaTreeType type);
gboolean rubrica_lib_extract_net_data(GtkTreeModel *model, GtkTreePath *path,
				      GtkTreeIter *iter, gpointer data);
gboolean rubrica_lib_extract_phone_data(GtkTreeModel *model, GtkTreePath *path,
					GtkTreeIter *iter, gpointer data);



RubricaTelephoneType rubrica_lib_telephone2int(gchar *tipo);
gchar *rubrica_lib_fetch(gchar *path);

void rubrica_lib_debug_net_list(GList *lst);
void rubrica_lib_debug_tel_list(GList *lst);


#endif
