/*
*  Rubrica
*  file: sort.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>

#include "rubrica.h"
#include "sort.h"

Ordinamento ordinamento;

/*  ordinamento delle schede */

void rubrica_init_ordinamento(void)
{
  ordinamento.modo  = ALFABETICO;
  ordinamento.tipo  = CRESCENTE;
  ordinamento.campo = SCHEDA;
}

/* 
   alfa:
   Confronta i due elementi a e b
   
    ordinamento  | a == b | a < b | a > b | a == NULL | b == NULL |
   +-------------+------------------------+-----------+-----------+
   |crescente    |   0    |   -1  |   1   |    -1     |     1     |  valore ritornato
   +-------------+--------+-------+-------+-----------+-----------+
   |decrescente  |   0    |    1  |  -1   |     1     |    -1     |
   +-------------+--------+-------+-------+-----------+-----------+
*/
gint alfa(gconstpointer a, gconstpointer b)
{
  RubricaItem *p, *s;
  gchar *str1, *str2;

  p = (RubricaItem *)a;  /* prima */
  s = (RubricaItem *)b;  /* seconda */

  switch(ordinamento.campo)
    {
    case SCHEDA:
      str1 = p->card;
      str2= s->card;
      break;
    case NOME:
      str1 = p->personal.first;
      str2 = s->personal.first;      
      break;
    case COGNOME:
      str1 = p->personal.last;
      str2 = s->personal.last;
      break;
    default:
      str1 = p->card;
      str2 = s->card;
      break;
    }

  if (!str1 && !str2)
    return 0;

  if (ordinamento.tipo == CRESCENTE)
    {
      if (!str1)
	return -1;

      if (!str2)
	return 1;

      if (strcmp(str1,str2) < 0)
	return -1;
      if (strcmp(str1,str2) > 0)
	return 1;
    }

  if (ordinamento.tipo == DECRESCENTE)
    {
      if (!str1)
	return 1;
      
      if (!str2)
	return -1;
      
      if (strcmp(str1,str2) > 0)
	return -1;
      if (strcmp(str1,str2) < 0)
	return 1;
    }  

  /* a == b */
  return 0;
}

gint creation_date(gconstpointer a, gconstpointer b)
{
  RubricaItem *p, *s;
  time_t t1, t2;

  p = (RubricaItem *)a;  /* prima */
  s = (RubricaItem *)b;  /* seconda */

  t1 = p->created;
  t2 = s->created;
  
  if (ordinamento.tipo == CRESCENTE)
    {
      if (t1 > t2)
	return 1;
      if (t1 < t2)
	return -1;
    }

  if (ordinamento.tipo == DECRESCENTE)
    {
      if (t1 > t2)
	return -1;
      if (t1 < t2)
	return 1;      
    }  
  
  /* a == b */
  return 0;
}

gint changing_date(gconstpointer a, gconstpointer b)
{
  RubricaItem *p, *s;
  time_t t1, t2;

  p = (RubricaItem *)a;  /* prima */
  s = (RubricaItem *)b;  /* seconda */

  t1 = p->last_change;
  t2 = s->last_change;
  
  if (ordinamento.tipo == CRESCENTE)
    {
      if (t1 > t2)
	return 1;
      if (t1 < t2)
	return -1;
    }

  if (ordinamento.tipo == DECRESCENTE)
    {
      if (t1 > t2)
	return -1;
      if (t1 < t2)
	return 1;      
    }  
  
  /* a == b */
  return 0;
}

Ordinamento rubrica_get_ordinamento(void)
{
  return (Ordinamento) ordinamento;
}

void  rubrica_ordinamento_set_campo(ORDINARE n)
{
  ordinamento.campo = (ORDINARE) n;
}

void  rubrica_ordinamento_set_alfabetico()
{
  ordinamento.modo = ALFABETICO;	
}

void  rubrica_ordinamento_set_creation_date()
{
  ordinamento.modo = DATA_CREAZIONE;
}

void  rubrica_ordinamento_set_modification_date()
{
  ordinamento.modo = DATA_MODIFICA;
}

void  rubrica_ordinamento_set_crescente()
{
  ordinamento.tipo = CRESCENTE;
}

void  rubrica_ordinamento_set_decrescente()
{
  ordinamento.tipo = DECRESCENTE;
}


void rubrica_riordina_lista()
{
  Rubrica *rubrica;
  GtkWidget *cl;
  gint n;
  gint i = 0;

/*  cl = rubrica_get_clist();
  gtk_clist_clear(GTK_CLIST(cl));

  n  = rubrica_get_items();
  rubrica = rubrica_get_rubrica();
*/

  /* reorder list alphabetically */
  if(ordinamento.modo == ALFABETICO)
    {
//      rubrica = g_list_sort(rubrica, (GCompareFunc) alfa);
    }

  /* reorder list by creation date */
  if (ordinamento.modo == DATA_CREAZIONE)
    {
//      rubrica = g_list_sort(rubrica, (GCompareFunc) creation_date);
    }

  /* reorder list by last change date */
  if (ordinamento.modo == DATA_MODIFICA)
    {
//      rubrica = g_list_sort(rubrica, (GCompareFunc) changing_date);
    }
  
  for(i=0; i < n; i++)
    {
      gchar *buffer[13];
      RubricaItem *r;

      r = rubrica_get_nth_item(rubrica, i);
      if (r)
	{
/*
	  gint pos;
	  gchar *web;
	  gchar *email;
	  gchar *telephone;
	  GList *l;

	  if (r->net.nweb > 0)
	    {
	      l = r->net.web_list;
	      web = g_strdup(l->data);
	    }
	  else
	    web = "";
	  
	  if (r->net.nmail > 0)
	    {
	      l = r->net.mail_list;
	      email = g_strdup(l->data);
	    }
	  else 
	    email = "";

	  if (r->telephone.ntel > 0)
	    {
	      TelNum *tn;

	      l = r->telephone.tel_list;
	      tn = l->data;

	      telephone = g_strdup(tn->number);
	    }
	  else
	    telephone = "";
	  
	  buffer[0] = "";                    // cella per l'eventuale pixmap
	  buffer[1] = (r->card                ? g_strdup(r->card)              : "");
	  buffer[2] = (r->personal.first_name ? g_strdup(r->personal.first_name) : "");
	  buffer[3] = (r->personal.last_name  ? g_strdup(r->personal.last_name)  : "");
	  buffer[4] = (r->personal.profession ? g_strdup(r->personal.profession) : "");
	  buffer[5] = (r->address.city        ? g_strdup(r->address.city)        : "");
	  buffer[6] = (r->address.country     ? g_strdup(r->address.country)     : "");
	  buffer[7] = web;
	  buffer[8] = email;
	  buffer[9] = telephone;
	  buffer[10] = (r->company.name   ? g_strdup(r->company.name)   : "");
	  buffer[11] = (r->work.assigment ? g_strdup(r->work.assigment) : "");
	  buffer[12] = NULL;
	  
	  pos = gtk_clist_insert(GTK_CLIST(cl), n, buffer);
	  gtk_clist_set_row_data(GTK_CLIST(cl), pos, r);
*/
	}
    }
}






